@@ -16,7 +16,7 @@ class TestWizard(ProteusTestCase):
 
     def test_translation_export(self):
         Lang = Model.get('ir.lang')
-        Module = Model.get('ir.module.module')
+        Module = Model.get('ir.module')
         translation_export = Wizard('ir.translation.export')
         translation_export.form.language, = Lang.find([('code', '=', 'en_US')])
         translation_export.form.module, = Module.find([('name', '=', 'ir')])
@@ -40,3 +40,9 @@ class TestWizard(ProteusTestCase):
 
         self.assert_(User.find([('name', '=', 'Foo')]))
         self.assertFalse(User.find([('name', '=', 'Bar')]))
+
+    def test_translation_update(self):
+        print_model_graph = Wizard('ir.translation.update')
+        self.assertEqual(len(print_model_graph.actions), 0)
+        print_model_graph.execute('update')
+        self.assertEqual(len(print_model_graph.actions), 1)
